/**
 * - Digitalis Internal Framework v2.0 -
 *
 * (C) 2007, Digitalis Informatica.
 *
 * Distribuicao e Gestao de Informatica, Lda. Estrada de Paco de Arcos num.9 - Piso -1 2780-666 Paco de Arcos Telefone:
 * (351) 21 4408990 Fax: (351) 21 4408999 http://www.digitalis.pt
 */
package pt.digitalis.utils.bytecode.holders;

import java.util.ArrayList;
import java.util.List;

import javassist.bytecode.annotation.AnnotationMemberValue;
import javassist.bytecode.annotation.ArrayMemberValue;
import javassist.bytecode.annotation.MemberValue;
import pt.digitalis.utils.CodeGenUtil4Javassist;
import pt.digitalis.utils.inspection.exception.ResourceNotFoundException;

/**
 * An helper class that helps to manage the temporary annotation member values in the code generation process. Provides
 * managing methods and cache the details read from the member.
 *
 * @author Pedro Viegas <a href="mailto:pviegas@digitalis.pt">pviegas@digitalis.pt</a>
 * @author Rodrigo Gonalves <a href="mailto:rgoncalves@digitalis.pt">rgoncalves@digitalis.pt</a>
 * @created Sep 23, 2007
 *
 */
public class AnnotationMemberValueHolder {

    /** The member value object. */
    private MemberValue memberValue;

    /**
     * Constructs a new holder from a received a member value object.
     *
     * @param memberValue
     *            the member value to manage
     */
    public AnnotationMemberValueHolder(MemberValue memberValue)
    {
        this.memberValue = memberValue;
    }

    /**
     * Returns the member value as a String.
     *
     * @return a String representation of the memberValue
     */
    public String toString()
    {
        return CodeGenUtil4Javassist.convertMemberValueToString(memberValue);
    }

    /**
     * Returns the member value as a Boolean.
     *
     * @return a Boolean representation of the memberValue
     */
    public Boolean toBoolean()
    {
        return CodeGenUtil4Javassist.convertMemberValueToBoolean(memberValue);
    }

    /**
     * Returns the member value as a Boolean.
     *
     * @param attribute
     *            the attribute holder for the inner annotations
     *
     * @return a Boolean representation of the memberValue
     */
    public List<AnnotationHolder> toAnnotationList(AttributeHolder attribute)
    {
        List<AnnotationHolder> results = new ArrayList<AnnotationHolder>();

        ArrayMemberValue arrayValue = (ArrayMemberValue) getManagedAnnotationValue();

        for (MemberValue annotationValue: arrayValue.getValue())
            results.add(new AnnotationHolder(attribute, ((AnnotationMemberValue) annotationValue).getValue(), true));

        return results;
    }

    /**
     * Returns the member value as a Boolean.
     *
     * @param clazz
     *            the class holder for the inner annotations
     *
     * @return a Boolean representation of the memberValue
     * @throws ResourceNotFoundException
     */
    public List<AnnotationHolder> toAnnotationList(ClassHolder clazz) throws ResourceNotFoundException
    {
        List<AnnotationHolder> results = new ArrayList<AnnotationHolder>();

        ArrayMemberValue arrayValue = (ArrayMemberValue) getManagedAnnotationValue();

        for (MemberValue annotationValue: arrayValue.getValue())
            results.add(new AnnotationHolder(clazz, ((AnnotationMemberValue) annotationValue).getValue(), true));

        return results;
    }

    /**
     * Returns the member value from an enumeration value as a String.
     *
     * @return a String representation of the enumeration memberValue
     */
    public String enumValuetoString()
    {
        if (memberValue != null)
            return CodeGenUtil4Javassist.getEnumValue(memberValue);
        else
            return null;
    }

    /**
     * Returns the annotation value object.
     *
     * @return the annotation value object
     */
    public MemberValue getManagedAnnotationValue()
    {
        return this.memberValue;
    }

}
